<cfcomponent displayname="Time Controller for Error Handling">
	<cfscript>
		//This component will assist with Time based error handling
		//you should initialize using the init function and passing the number
		//of iterations that would indicate a good average as well as the maximum iterations expected
		//You  can reset the maximum iterations even later with a setter function setMaxIterations()
		// Bilal Soylu  11-05-2009
		// (c) 2009 Apache Creative Commons License v. 3		
		
		//first get start time
		this.dteStart = Now();		
		
		//initialize other variables needed
		this.intTimerStart = 0;
		this.intTimerEnd = 0;
		this.intProcessingTime = 0;
		this.intAvgRecordTime = 0;
		this.intSumRecordTime = 0;
		this.minIterations =10;
		this.maxIterations =0; //the maximum number of times we expect to run through the time capture
		this.currentIteration =0;
		this.blnValidEstimate = false;
	</cfscript>
	
	<!--- init function  --->
	<cffunction name="init" access="public" hint="initializes the min/max iterations if needed">
		<cfargument name="maxIterations" type="numeric" hint="the max number of iteration required before timing is deemed accurate">
		<cfargument name="minIterations" type="numeric" hint="the number of iteration required before timing is deemed accurate">
		
		<cfset this.intTimeOut = fGetTimeout()>
		<cfset this.dtePageTimeOut = DateAdd("s",this.intTimeOut,this.dteStart)>
		
		<cfif IsDefined("Arguments.maxIterations")>
			<cfset this.maxIterations = Arguments.maxIterations>		
		</cfif>			
		<cfif IsDefined("Arguments.minIterations")>
			<cfset this.minIterations = Arguments.minIterations>		
		</cfif>	
		<cfreturn this>
	</cffunction>
	
	
	<cffunction name="setMaxIterations" access="public" hint="set max iterations for timing">
		<cfargument name="maxIterations" required="Yes" type="numeric" hint="the max number of iteration that will be timed">
		
		<cfset this.maxIterations= Arguments.maxIterations>
		
	</cffunction>

	<!--- start and end functions  --->
	<cffunction name="start" access="public" hint="starts a time capture process">
		<cfset this.intTimerStart = GetTickCount()>
	</cffunction>	
	
	
	<cffunction name="end" access="public" hint="ends a time capture process">
		<cfset this.intTimerEnd = GetTickCount()>
		<cfset this.currentIteration++>
		<cfset fCalcEstimates()>
	</cffunction>		
	
	<!--- getters   --->
	<cffunction name="getFinishTime" access="public" hint="returns finish time">
		<cfargument name="format" default="yes" type="boolean" required="no" hint="should formatting be applied">
		
		<cfset var strReturn = DateAdd("s",this.intProcessingTime,Now())>		
		<cfif Arguments.format>
			<cfset strReturn = TimeFormat(strReturn,"HH:mm:ss")>
		</cfif>
		<cfreturn strReturn>
	</cffunction>	
	
	<cffunction name="getFinishSeconds" access="public" hint="returns finish in remaining seconds">
		<cfargument name="format" default="yes" type="boolean" required="no" hint="should formatting be applied">
		
		<cfset var strReturn = this.intProcessingTime>		
		<cfif Arguments.format>
			<cfset strReturn = DecimalFormat(strReturn)>
		</cfif>
		<cfreturn strReturn>
	</cffunction>	
	
	<cffunction name="getPageTimeOut" access="public" hint="returns expected page timeout">
		<cfargument name="format" default="yes" type="boolean" required="no" hint="should formatting be applied">
		
		<cfset var strReturn = this.dtePageTimeOut>		
		<cfif Arguments.format>
			<cfset strReturn = TimeFormat(strReturn,"HH:mm:ss")>
		</cfif>
		<cfreturn strReturn>
	</cffunction>			
	
	
	<!--- conditional functions  --->	
	<cffunction name="WillTimeout" access="public" returntype="boolean" hint="determines whether time out could occur">
		<cfset var blnReturn=false>
		<cfif this.blnValidEstimate AND DateAdd("s",this.intProcessingTime,Now()) GT this.dtePageTimeOut>
			<cfset blnReturn=true>		
		</cfif>
		<cfreturn blnReturn>
	</cffunction>	

	<cffunction name="HasValidAverage" access="public" returntype="boolean" hint="determines whether the calculations have passed threshold for validity">
		<cfreturn this.blnValidEstimate>
	</cffunction>
	
	
	<!--- Private Functions (bsoylu 11-11-2009) --->
	
	<cffunction name="fCalcEstimates" access="private" hint="update averages and time estimates">	
		<cfset var intRecordTime = (this.intTimerEnd - this.intTimerStart) >
		<cfset this.intSumRecordTime = this.intSumRecordTime + intRecordTime >
		<cfset this.intAvgRecordTime = this.intSumRecordTime / this.currentIteration >
		<cfset this.intProcessingTime = (((1000000 - this.currentIteration) * this.intAvgRecordTime) / 1000) >	
		<cfif not this.blnValidEstimate AND this.currentIteration GT this.minIterations>
			<cfset this.blnValidEstimate = true>			
		</cfif>
	</cffunction>
	
	<cffunction name="fGetTimeout" access="private" returntype="numeric" hint="gets the timeout from coldfusion w/o needing API password">
		<!--- neo-runtime.xml contains request timeout info (bsoylu 10-25-2009) --->
		<cfset var strFile="#Server.ColdFusion.RootDir#\lib\neo-runtime.xml">
		<cfset var strFI = "">
		<cfset var arrCFDebugConfig = "">
		<cfset var intTimeOut = -1>
		
		<!--- read the  config file and convert to cf object (bsoylu 10-25-2009) --->
		<cffile action="READ" file="#strFile#" variable="strFI">		
		<cfwddx action="WDDX2CFML" input="#strFI#" output="arrRuntime">	
		
		<!--- set the return value from array (bsoylu 11-10-2009) --->
		<cfset intTimeOut=arrRuntime[10].timeoutRequestTimeLimit>
	
		<cfreturn intTimeOut>
	</cffunction>	

</cfcomponent>
